@@ -49,6 +49,13 @@ class MergedSQLQueryResolver(SingleSQLQueryResolver):
         for k in self.query_dimensions.keys():
             if k not in join_hashes:
                 join_hashes.append(k)
+
+        join_hash_readability_lookup = {
+            j: f"{j.split('__')[0]}__cte_subquery_{i}" for i, j in enumerate(sorted(join_hashes))
+        }
+        print(join_hash_readability_lookup)
+
+        readable_join_hashes = []
         for join_hash in join_hashes:
             metrics = [f.id() for f in self.query_metrics.get(join_hash, [])]
             dimensions = [f.id() for f in self.query_dimensions.get(join_hash, [])]
@@ -66,15 +73,19 @@ class MergedSQLQueryResolver(SingleSQLQueryResolver):
                 **kws,
             )
             query = resolver.get_query(semicolon=False)
-            queries_to_join[join_hash] = query
+            readable_hash = join_hash_readability_lookup[join_hash]
+            readable_join_hashes.append(readable_hash)
+            queries_to_join[readable_hash] = query
 
+        readable_metrics = {join_hash_readability_lookup[k]: m for k, m in self.query_metrics.items()}
+        readable_dimensions = {join_hash_readability_lookup[k]: d for k, d in self.query_dimensions.items()}
         query_config = {
             "merged_metrics": self.merged_metrics,
-            "query_metrics": self.query_metrics,
-            "query_dimensions": self.query_dimensions,
+            "query_metrics": readable_metrics,
+            "query_dimensions": readable_dimensions,
             "having": self.having,
             "queries_to_join": queries_to_join,
-            "join_hashes": list(sorted(join_hashes)),
+            "join_hashes": list(sorted(readable_join_hashes)),
             "query_type": resolver.query_type,
             "limit": self.limit,
             "project": self.project,
@@ -118,7 +129,7 @@ class MergedSQLQueryResolver(SingleSQLQueryResolver):
             join_group_hash = self._join_hash_key(field)
             if field.canon_date is None:
                 raise QueryError(
-                    f"Could not find a date field associated with metric {ref_field.name} "
+                    f"Could not find a date field associated with metric {field.name} "
                     "in the project. \n\nMake sure you have this defined either in the view "
                     "with the property 'default_date' or on the metric under 'canon_date'"
                 )
@@ -175,7 +186,14 @@ class MergedSQLQueryResolver(SingleSQLQueryResolver):
                                 self.query_dimensions[mapping_info["from_join_hash"]].append(ref_field)
                             else:
                                 canon_date = ref_field.canon_date.replace(".", "_")
-                                key = f"{canon_date}__{mapping_info['to_join_hash']}"
+                                existing_join_hashes = (
+                                    join_hash
+                                    for join_hash in used_join_hashes
+                                    if mapping_info["to_join_hash"] in join_hash
+                                )
+                                join_hash = next(existing_join_hashes, None)
+                                default_join_hash = f"{canon_date}__{mapping_info['to_join_hash']}"
+                                key = join_hash if join_hash else default_join_hash
                                 self.query_dimensions[key].append(ref_field)
 
                 if not_in_metrics:
@@ -201,7 +219,7 @@ class MergedSQLQueryResolver(SingleSQLQueryResolver):
             field = self.project.get_field(where["field"])
             is_canon_date = any(f"{field.view.name}.{field.name}" == d for d in metric_canon_dates)
             dimension_group = field.dimension_group
-            join_group_hash = self.project.join_graph.join_graph_hash(field.view.name)
+            join_group_hash = self._join_hash_key(field)
             added_filter = {join_hash: False for join_hash in unique_keys}
             for join_hash in unique_keys:
                 join_hash_with_canon_date = f"{field.view.name}_{field.name}__{join_group_hash}"
