@@ -38,12 +38,11 @@ class MetricsLayerDesign:
         required_views = self.required_views()
 
         self._join_subgraph = self.project.join_graph.subgraph(required_views)
-
         try:
             ordered_view_pairs = self.determine_join_order(required_views)
         except networkx.exception.NetworkXNoPath:
             raise JoinError(
-                f"There was no join path between the views: {required_views}. "
+                f"There was no join path between the views: {list(sorted(required_views))}. "
                 "Check the identifiers on your views and make sure they are joinable."
             )
 
